GOVERNMENTS = {
    0:	{
	'name':		"Feudal",
	'morale':       0,
    },
    1:	{
	'name':		"Feudal 2",
	'morale':       0,
    },
    2:	{
	'name':		"Dictatorship",
	'morale':       -20,                    # -20%
    },
    3:	{
	'name':		"Imperium",
	'morale':       0,
    },
    4:	{
	'name':		"Democracy",
	'morale':       0,
    },
    5:	{
	'name':		"Democracy 2",
	'morale':       0,
    },
    6:	{
	'name':		"Unification",
	'morale':       0,
    },
    7:	{
	'name':		"Galactic Unification",
	'morale':       0,
    },
}
