SMALL_SHIP	= 0
MEDIUM_SHIP	= 1
LARGE_SHIP	= 2
HUGE_SHIP	= 3
TITAN_SHIP	= 4
DOOMSTAR_SHIP	= 5


// values for ship type field:
COMBAT_SHIP_TYPE	= 0
COLONY_SHIP_TYPE	= 1
TRANSPORT_SHIP_TYPE	= 2
#UNUSED                   3
OUTPOST_SHIP_TYPE	= 4
