FARMER		= 0x02
SCIENTIST	= 0x03
WORKER		= 0x82

GOVERMENT_FEUDAL        = 0
GOVERMENT_FEUDAL2       = 1
GOVERMENT_DICTATORSHIP  = 2
GOVERMENT_IMPERIUM	= 3
GOVERMENT_DEMOCRACY     = 4
GOVERMENT_UNIFICATION   = 6

PLANET_LOW_G	= 0
PLANET_NORMAL_G	= 1
PLANET_HEAVY_G	= 2

TERRAIN_TOXIC           = 0
TERRAIN_RADIATED        = 1
TERRAIN_BARED           = 2
TERRAIN_DESERT          = 3
TERRAIN_TUNDRA          = 4
TERRAIN_OCEAN           = 5
TERRAIN_SWAMP           = 6
TERRAIN_ARID            = 7
TERRAIN_TERRAN          = 8
TERRAIN_GAIA            = 9

MINERAL_ULTRAPOOR       = 0
MINERAL_POOR            = 1
MINERAL_ABUNDANT        = 2
MINERAL_RICH            = 3
MINERAL_ULTRARICH       = 4

SPECIAL_ARTIFACTS       = 10

BUILD_TRADE_GOODS	= 0xFE

TECH_ANTI_MISSILE_ROCKETS       =  13
TECH_COLONY_BASE                =  39
TECH_COLONY_SHIP                =  40
TECH_REINFORCED_HULL            =  56
TECH_BIOSPHERES                 =  61
TECH_FIGHTER_BAYS               =  66
TECH_FREIGHTERS                 =  69

TECH_HEIGHTENED_INTELLIGENCE	=  83
TECH_HYDROPONIC_FARM            =  87
TECH_TRANSPORT                  = 188
TECH_MICROLITE_CONSTRUCTION	= 107
TECH_OUTPOST_SHIP               = 108
TECH_NANO_DISASSEMBLERS		= 112
TECH_NEURAL_SCANNER             = 113
TECH_NUCLEAR_BOMB               = 119
TECH_PSIONICS                   = 146
TECH_SPACE_ACADEMY              = 163
TECH_SPY_NETWORK                = 165   # 166 ?
TECH_TELEPATHIC_TRAINING        = 181
TECH_VIRTUAL_REALITY_NETWORK	= 194

TECH_GROUP_CONSTRUCTIONS        =  1
TECH_GROUP_SPIES                =  2
TECH_GROUP_COLONY               =  3
TECH_GROUP_SHIP_EQUIPMENT       =  4
TECH_GROUP_GROUND_COMBAT        =  5
TECH_GROUP_MISCELANEOUS         =  6

TECH_REVIEW = {
    0: [],
    TECH_GROUP_CONSTRUCTIONS: [TECH_TRANSPORT, TECH_OUTPOST_SHIP, TECH_FREIGHTERS, TECH_COLONY_SHIP, TECH_COLONY_BASE]
}
