RESEARCH = {
    'construction':     {
        'index':            0,
        'start_area':       4,
    },
    'power':            {
        'index':            1,
        'start_area':       55,
    },
    'chemistry':        {
        'index':            2,
        'start_area':       22,
    },
    'sociology':        {
        'index':            3,
        'start_area':       10,
    },
    'computers':        {
        'index':            4,
        'start_area':       28,
    },
    'biology':          {
        'index':            5,
        'start_area':       18,
    },
    'physics':          {
        'index':            6,
        'start_area':       57,
    },
    'force_fields':     {
        'index':            7,
        'start_area':       7,
    },
}

RESEARCH_AREAS = {
    1:	{
	'name':		"Advanced Biology",
	'cost':		400,
	'next':		34,
    },
    2:	{
	'name':		"Advanced Chemistry",
	'cost':		650,
	'next':		47,
    },
    3:	{
	'name':		"Advanced Construction",
	'cost':		150,
	'next':		21,
    },
    4:	{
	'name':		"Advanced Engineering",
	'cost':		80,
	'next':		3,
    },
    5:	{
	'name':		"Advanced Fusion",
	'cost':		250,
	'next':		41,
    },
    6:	{
	'name':		"Advanced Governments",
	'cost':		4500,
	'next':		32,
    },
    7:	{
	'name':		"Advanced Magnetism",
	'cost':		250,
	'next':		36,
    },
    8:	{
	'name':		"Advanced Manufacturing",
	'cost':		1500,
	'next':		11,
    },
    9:	{
	'name':		"Advanced Metallurgy",
	'cost':		250,
	'next':		2,
    },
    10:	{
	'name':		"Military Tactics",
	'cost':		150,
	'next':		73,
    },
    11:	{
	'name':		"Advanced Robotics",
	'cost':		2000,
	'next':		67,
    },
    12:	{
	'name':		"Teaching Methods",
	'cost':		2000,
	'next':		6,
    },
    13:	{
	'name':		"Anti-Matter Fission",
	'cost':		2000,
	'next':		46,
    },
    14:	{
	'name':		"Artifical Consciousness",
	'cost':		1500,
	'next':		25,
    },
    15:	{
	'name':		"Artifical Intelligence",
	'cost':		400,
	'next':		60,
    },
    16:	{
	'name':		"Artifical Gravity",
	'cost':		1150,
	'next':		65,
    },
    17:	{
	'name':		"Artifical Life",
	'cost':		4500,
	'next':		70,
    },
    18:	{
	'name':		"Astro Biology",
	'cost':		80,
	'next':		1,
    },
    19:	{
	'name':		"Astro Construction",
	'cost':		1150,
	'next':		8,
    },
    20:	{
	'name':		"Astro Engineering",
	'cost':		400,
	'next':		62,
    },
    21:	{
	'name':		"Capsule Construction",
	'cost':		250,
	'next':		20,
    },
    22:	{
	'name':		"Chemistry",
	'cost':		50,
	'next':		9,
    },
    23:	{
	'name':		"Cold Fusion",
	'cost':		80,
	'next':		5,
    },
    24:	{
	'name':		"Cybertechnics",
	'cost':		3500,
	'next':		33,
    },
    25:	{
	'name':		"Cybertronics",
	'cost':		2750,
	'next':		24,
    },
    26:	{
	'name':		"Distortion Fields",
	'cost':		3500,
	'next':		61,
    },
    27:	{
	'name':		"Electromagnetic Refraction",
	'cost':		1500,
	'next':		72,
    },
    28:	{
	'name':		"Electronics",
	'cost':		50,
	'next':		56,
    },
    30:	{
	'name':		"Evolutionary Genetics",
	'cost':		2750,
	'next':		17,
    },
    31:	{
	'name':		"Fusion Physics",
	'cost':		150,
	'next':		66,
    },
    32:	{
	'name':		"Galactic Economics",
	'cost':		6000,
	'next':		82,
    },
    33:	{
	'name':		"Galactic Networking",
	'cost':		4500,
	'next':		49,
    },
    34:	{
	'name':		"Genetic Engineering",
	'cost':		900,
	'next':		35,
    },
    35:	{
	'name':		"Genetic Mutations",
	'cost':		1150,
	'next':		44,
    },
    36:	{
	'name':		"Gravitic Fields",
	'cost':		650,
	'next':		45,
    },
    37:	{
	'name':		"High Energy Distribution",
	'cost':		3500,
	'next':		38,
    },
    38:	{
	'name':		"Hyper Dimensioanl Fission",
	'cost':		4500,
	'next':		40,
    },
    39:	{
	'name':		"Hyper Dimensional Physics",
	'cost':		6000,
	'next':		69,
    },
    40:	{
	'name':		"Interphased Fission",
	'cost':		10000,
	'next':		76,
    },
    41:	{
	'name':		"Ion Fission",
	'cost':		900,
	'next':		13,
    },
    42:	{
	'name':		"Superscalar Construction",
	'cost':		6000,
	'next':		58,
    },
    43:	{
	'name':		"Macro Economics",
	'cost':		1150,
	'next':		12,
    },
    44:	{
	'name':		"Macro Genetics",
	'cost':		1500,
	'next':		30,
    },
    45:	{
	'name':		"Magneto Gravitics",
	'cost':		900,
	'next':		27,
    },
    46:	{
	'name':		"Matter Energy Conversion",
	'cost':		2750,
	'next':		37,
    },
    47:	{
	'name':		"Molecular Compression",
	'cost':		1150,
	'next':		53,
    },
    48:	{
	'name':		"Molecular Control",
	'cost':		10000,
	'next':		80,
    },
    49:	{
	'name':		"Moleculartronics",
	'cost':		6000,
	'next':		81,
    },
    50:	{
	'name':		"Molecular Manipulation",
	'cost':		4500,
	'next':		48,
    },
    51:	{
	'name':		"Multi-Dimensional Physics",
	'cost':		4500,
	'next':		39,
    },
    52:	{
	'name':		"Multi-Phased Physics",
	'cost':		2000,
	'next':		59,
    },
    53:	{
	'name':		"Nano Technology",
	'cost':		2000,
	'next':		50,
    },
    54:	{
	'name':		"Neutrino Physics",
	'cost':		900,
	'next':		16,
    },
    55:	{
	'name':		"Nuclear Fission",
	'cost':		50,
	'next':		23,
    },
    56:	{
	'name':		"Optronic",
	'cost':		150,
	'next':		15,
    },
    57:	{
	'name':		"Physics",
	'cost':		50,
	'next':		31,
    },
    58:	{
	'name':		"Planetoid Construction",
	'cost':		7500,
	'next':		78,
    },
    59:	{
	'name':		"Plasma Physics",
	'cost':		3500,
	'next':		51,
    },
    60:	{
	'name':		"Positronics",
	'cost':		900,
	'next':		14,
    },
    61:	{
	'name':		"Quantum Fileds",
	'cost':		4500,
	'next':		71,
    },
    62:	{
	'name':		"Robotics",
	'cost':		650,
	'next':		63,
    },
    63:	{
	'name':		"Servo Mechanics",
	'cost':		900,
	'next':		19,
    },
    64:	{
	'name':		"Subspace Fields",
	'cost':		2750,
	'next':		26,
    },
    65:	{
	'name':		"Subspace Physics",
	'cost':		1500,
	'next':		52,
    },
    66:	{
	'name':		"Tachyon Physics",
	'cost':		250,
	'next':		54,
    },
    67:	{
	'name':		"Tectonic Engineering",
	'cost':		3500,
	'next':		42,
    },
    68:	{
	'name':		"Temporal Fields",
	'cost':		15000,
	'next':		79,
    },
    69:	{
	'name':		"Temporal Physics",
	'cost':		15000,
	'next':		77,
    },
    70:	{
	'name':		"Trans Genetics",
	'cost':		7500,
	'next':		75,
    },
    71:	{
	'name':		"Transwarp Fields",
	'cost':		7500,
	'next':		68,
    },
    72:	{
	'name':		"Warp Fields",
	'cost':		2000,
	'next':		64,
    },
    73:	{
	'name':		"Xeno Relations",
	'cost':		650,
	'next':		43,
    },
    75:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    76:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    77:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    78:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    79:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    80:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    81:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
    82:	{
	'name':		"Hyper-advanced",
	'cost':		25000,
	'next':		0,
    },
}
